@@ -15,6 +15,9 @@ static __thread char **con_array;
 static __thread int con_array_size;
 static __thread int con_array_used;
 
+static pthread_once_t once = PTHREAD_ONCE_INIT;
+static pthread_key_t destructor_key;
+
 static int add_array_elt(char *con)
 {
 	if (con_array_size) {
@@ -282,11 +285,24 @@ void matchpathcon_filespec_destroy(void)
 	fl_head = NULL;
 }
 
+static void matchpathcon_thread_destructor(void __attribute__((unused)) *ptr)
+{
+	matchpathcon_fini();
+}
+
+static void matchpathcon_init_once(void)
+{
+	__selinux_key_create(&destructor_key, matchpathcon_thread_destructor);
+}
+
 int matchpathcon_init_prefix(const char *path, const char *subset)
 {
 	if (!mycanoncon)
 		mycanoncon = default_canoncon;
 
+	__selinux_once(once, matchpathcon_init_once);
+	__selinux_setspecific(destructor_key, (void *)1);
+
 	options[SELABEL_OPT_SUBSET].type = SELABEL_OPT_SUBSET;
 	options[SELABEL_OPT_SUBSET].value = subset;
 	options[SELABEL_OPT_PATH].type = SELABEL_OPT_PATH;
@@ -305,6 +321,8 @@ int matchpathcon_init(const char *path)
 
 void matchpathcon_fini(void)
 {
+	free_array_elts();
+
 	if (hnd) {
 		selabel_close(hnd);
 		hnd = NULL;
